package model.plateau;

import javafx.beans.property.*;
import model.CellulesVivantes;
import model.cellule.Cellule;
import model.cellule.créateur.CreateurCellule;

import java.util.List;

public class Plateau implements PrototypePlateau{
	private CreateurCellule createurCellule;
	private IntegerProperty colone = new SimpleIntegerProperty();
		public int getColone() { return colone.get();}
		public void setColone(int valeur) { colone.set(valeur);	resetGrille(valeur, getLigne());}
		public IntegerProperty coloneProperty() { return colone; }

	private IntegerProperty ligne = new SimpleIntegerProperty();
		public int getLigne() { return ligne.get(); }
		public void setLigne(int valeur ) { ligne.set(valeur); resetGrille(getColone(), valeur);}
		public IntegerProperty ligneProperty() { return ligne; }

	//private ObservableList<List<Cellule>> grilleObs = FXCollections.observableArrayList();
	private ListProperty<List<Cellule>> grille = new SimpleListProperty<>();
		public ListProperty<List<Cellule>> getGrille() { return (ListProperty<List<Cellule>>) grille.get(); }
		public void setGrille(ListProperty<List<Cellule>> cells) {grille.set(cells);}
		public ReadOnlyListProperty grilleProperty() { return grille;}
	private CellulesVivantes cellulesVivantes;

	public Cellule getCell(int x, int y) throws IllegalArgumentException{
		if(x < 0 || y < 0) {
			throw new IllegalArgumentException("X ou Y est inférieur à 0");
		}
		if(y >= getGrille().size()){
			throw new IllegalArgumentException("Y est trop grand !!!");
		}
		if(x >= getGrille().get(y).size()){
			throw new IllegalArgumentException("X est trop grand !!!");
		}
		return grille.get().get(y).get(x);
	}

	public void resetGrille(){
			resetGrille(getColone(), getLigne());
	}
	public void resetGrille(int colone, int ligne){
			setGrille(createurCellule.creerCellules(colone, ligne, cellulesVivantes));
	}

	public Plateau(){
			createurCellule = new CreateurCellule(0, 0);
			cellulesVivantes = new CellulesVivantes();
			setGrille(new SimpleListProperty<>());
		}
	public Plateau(int colone, int ligne) {
		this(colone, ligne, new CellulesVivantes());
	}
	public Plateau(int colone, int ligne, CellulesVivantes observer) {
		createurCellule = new CreateurCellule(colone, ligne);
		setLigne(ligne);
		setColone(colone);
		cellulesVivantes = observer;
		setGrille(createurCellule.creerCellules(cellulesVivantes));
	}

	public Plateau(int colone, int ligne, ListProperty<List<Cellule>> cellules)
	{
		this(colone, ligne);
		setGrille(cellules);
	}

	@Override
	public Plateau cloner() {
		return new Plateau(getColone(), getLigne(), getGrille());
	}

	public CellulesVivantes getCellulesVivantes() {
		return cellulesVivantes;
	}
}
